% eda09_10
% complex z-plane analysis of filters, case 1, highpass

% i used here as imaginary unit!
clear i;

% standard time/frequency setup
N=32;
Nf=N/2+1;
Dt=1.0;
tmax=Dt*(N-1); 
t=Dt*[0:N-1]'; 
fmax=1/(2.0*Dt);
Df=fmax/(N/2);
f=Df*[0:N/2,-N/2+1:-1]'; 
Dw=2*pi*Df;
w=Dw*[0:N/2,-N/2+1:-1]'; 

% a monomial filter, and its one root
F = [1, -1.1]';
zp = -F(1)/F(2);  % 0=F1+F2*z  so z=-F1/F2
disp(sprintf('zero at %f',zp));

% complex z-plane
Nz=128;
zmin=-2;
zmax=2;
dz=(zmax-zmin)/(Nz-1); 
zr=zmin+dz*[0:Nz-1]'; 
zi=zmin+dz*[0:Nz-1]';

% evaluate filter polynomial everywhere in complex z-plane
S = zeros(Nz,Nz);
for n = [1:Nz]
    for m = [1:Nz]
        S(m,n) = abs( F(1) + F(2)*(zr(n)+i*zi(m)) )^2;
    end
end

% plot amplitude of polynomial
figure(1);
clf;
subplot(1,2,1);
set(gca,'LineWidth',2);
hold on;
axis ij;
axis equal;
axis([zmin, zmax, zmin, zmax]);
bw=0.9*(256-linspace(0,255,256)')/256;
colormap([bw,bw,bw]);
range=max(max(S))-min(min(S));
if( range==0 )
    range=1;
end
imagesc( [zmin, zmax], [zmin, zmax], (S-min(min(S)))/range);
hold on;
xlabel('real z');
ylabel('imag z');
title('|S|^2');

% plot Fourier transform points on the unit circle
plot(0,0,'wo','LineWidth',2);
z=exp(-i*w*Dt);
plot(real(z),imag(z),'k+','LineWidth',2);

% plot zero of filter
plot(real(zp),imag(zp),'w*','LineWidth',2);

% plot frequency response of filter
subplot(1,2,2);
set(gca,'LineWidth',2);
hold on;
Sz = abs( F(1)+F(2)*z ) .^2;
axis( [0, f(Nf), 0, max(Sz)] );
plot(f(1:Nf),Sz(1:Nf),'k-','LineWidth',2);
plot(f(1:Nf),Sz(1:Nf),'ko','Linewidth',2);
xlabel('frequency, Hz');
ylabel('|S|^2');